/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include "nrf.h"
#include <stdbool.h>
#include <stdint.h>
#include "bsp.h"
#include "nrf_gpio.h"

// Peripheral channel assignments
#define PIN0_GPIOTE_CH      0
#define PIN0_PPI_CH_A       0
#define PIN0_PPI_CH_B       1
#define PIN0_TIMER_CC_NUM   0


// TIMER3 reload value. The frequency equals '16000000 / TIMER_RELOAD'
#define TIMER_RELOAD        16
// The timer CC register used to reset the timer. The timers on nRF51 have 4 CC registers.
#define TIMER_RELOAD_CC_NUM 3
// The pin toggles after TIMER_TOGGLE_VALUE number of ticks. (note that it is currently set up to also toggle after TIMER_RELOAD ticks.
#define TIMER_TOGGLE_VALUE 8


// This function initializes timer 0 with the following configuration:
// 24-bit, base frequency 16 MHz, auto clear on COMPARE3 match (CC3 = TIMER_RELOAD)
void timer_init()
{
    NRF_TIMER0->BITMODE                 = TIMER_BITMODE_BITMODE_24Bit << TIMER_BITMODE_BITMODE_Pos;
    NRF_TIMER0->PRESCALER               = 0;
    NRF_TIMER0->SHORTS                  = TIMER_SHORTS_COMPARE0_CLEAR_Msk << TIMER_RELOAD_CC_NUM;   // restart timer after RELOAD_CC_NUM matches
    NRF_TIMER0->MODE                    = TIMER_MODE_MODE_Timer << TIMER_MODE_MODE_Pos;
    NRF_TIMER0->CC[TIMER_RELOAD_CC_NUM] = TIMER_RELOAD;    
    NRF_TIMER0->CC[PIN0_TIMER_CC_NUM]   = TIMER_TOGGLE_VALUE;
}


// Starts TIMER0
void timer_start()
{
    NRF_TIMER0->TASKS_START = 1;
}


// This function sets up TIMER0, the PPI and the GPIOTE modules to configure a single channel
// Timer CC num, PPI channel nums and GPIOTE channel num is defined at the top of this file
void PIN0_init(uint32_t pinselect)
{  
    NRF_GPIOTE->CONFIG[PIN0_GPIOTE_CH] = GPIOTE_CONFIG_MODE_Task << GPIOTE_CONFIG_MODE_Pos | 
                                         GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos | 
                                         pinselect << GPIOTE_CONFIG_PSEL_Pos | 
                                         GPIOTE_CONFIG_OUTINIT_High << GPIOTE_CONFIG_OUTINIT_Pos;

    NRF_PPI->CH[PIN0_PPI_CH_A].EEP = (uint32_t)&NRF_TIMER0->EVENTS_COMPARE[PIN0_TIMER_CC_NUM];      //first  compare after TIMER_TOGGLE_VALUE  (event)
    NRF_PPI->CH[PIN0_PPI_CH_A].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[PIN0_GPIOTE_CH];              //first  compare after TIMER_TOGGLE_VALUE  (task)
    NRF_PPI->CH[PIN0_PPI_CH_B].EEP = (uint32_t)&NRF_TIMER0->EVENTS_COMPARE[TIMER_RELOAD_CC_NUM];    //second compare after TIMER_RELOAD_CC_NUM (event)
    NRF_PPI->CH[PIN0_PPI_CH_B].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[PIN0_GPIOTE_CH];              //second compare after TIMER_RELOAD_CC_NUM (task)
    
    NRF_PPI->CHENSET               = (1 << PIN0_PPI_CH_A) | (1 << PIN0_PPI_CH_B);
}


int main(void)
{
    // Initialize the timer
    timer_init();
    
    // Initialize pin toggle
    PIN0_init(LED_1);
    

    
    // Start the timer
    timer_start();

    while (true)
    {
        __WFI(); 
    }
}
